<?php

namespace InspireLabs\WooInpost;

use Exception;
use InspireLabs\WooInpost\admin\Alerts;
use InspireLabs\WooInpost\shipx\models\courier_pickup\ShipX_Dispatch_Order_Point_Address_Model;
use InspireLabs\WooInpost\shipx\models\courier_pickup\ShipX_Dispatch_Order_Point_Model;

/**
 * EasyPack API
 *
 * @author      WPDesk
 * @category    Admin
 * @package     EasyPack
 * @version     2.1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'InspireLabs\WooInpost\EasyPack_API' ) ) :

	class EasyPack_API {

		const API_URL_PRODUCTION_PL = 'https://api-shipx-pl.easypack24.net';

		const API_URL_SANDBOX_PL = 'https://sandbox-api-shipx-pl.easypack24.net';

		const API_URL_PRODUCTION_UK = 'https://api-shipx-uk.easypack24.net/v1/';

		const API_URL_SANDBOX_UK = 'https://sandbox-api-shipx-uk.easypack24.net/v1/';

		const ENVIRONMENT_PRODUCTION = 'production';

		const ENVIRONMENT_SANDBOX = 'sandbox';

		const COUNTRY_PL = 'PL';

		const COUNTRY_UK = 'GB';

		/**
		 * @var self
		 */
		protected static $instance;

		/**
		 * @var string
		 */
		private $environment;

		/**
		 * @var string
		 */
		private $country;

		/**
		 * @var string
		 */
		private $api_url;

		protected $token;

		protected $cache_period = DAY_IN_SECONDS;


		public function __construct() {
			$this->token = get_option( 'easypack_token' );
			$this->setupEnvironment();
		}

		private function setupEnvironment() {
			if ( 'sandbox' === get_option( 'easypack_api_environment' ) ) {
				$this->environment = self::ENVIRONMENT_SANDBOX;
			} else {
				$this->environment = self::ENVIRONMENT_PRODUCTION;
			}

			if ( self::COUNTRY_PL === $this->normalize_country_code_for_inpost(
					get_option( 'easypack_api_country' ) )
			) {
				$this->country = self::COUNTRY_PL;
			}

			if ( self::COUNTRY_UK === $this->normalize_country_code_for_inpost(
					get_option( 'easypack_api_country' )
				)
			) {
				$this->country = self::COUNTRY_UK;
			}

			$this->api_url            = $this->make_api_url();
		}

		/**
		 * @param string $country_code
		 *
		 * @return string
		 */
		public function normalize_country_code_for_geowidget( $country_code ) {
			return strtolower( $country_code );
		}

		/**
		 * @param string $country_code
		 *
		 * @return string
		 */
		public function normalize_country_code_for_inpost( $country_code ) {
			return strtoupper( $country_code );
		}

		/**
		 * @return bool
		 */
		public function is_sandbox_env() {
			return self::ENVIRONMENT_SANDBOX === $this->environment;
		}

		/**
		 * @return bool
		 */
		public function is_production_env() {
			return self::ENVIRONMENT_PRODUCTION === $this->environment;
		}

		/**
		 * @return bool
		 */
		public function is_uk() {
			return self::COUNTRY_UK === $this->country;
		}

		/**
		 * @return bool
		 */
		public function is_pl() {
			return self::COUNTRY_PL === $this->country;
		}

		public static function EasyPack_API() {
			if ( self::$instance === null ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * @param null $url
		 *
		 * @return null|string
		 */
		public function make_api_url( $url = null ) {

			if ( self::ENVIRONMENT_SANDBOX === $this->environment ) {
				if ( self::COUNTRY_PL === $this->country ) {
					$url = self::API_URL_SANDBOX_PL;
					$this->geo_widget_api_url
					     = 'https://sandbox-api-pl-points.easypack24.net/v1';
				}

				if ( self::COUNTRY_UK === $this->country ) {
					$url = self::API_URL_SANDBOX_UK;
					$this->geo_widget_api_url
					     = 'https://sandbox-api-uk-points.easypack24.net/v1';
				}
			}

			if ( self::ENVIRONMENT_PRODUCTION === $this->environment ) {
				if ( self::COUNTRY_PL === $this->country ) {
					$url = self::API_URL_PRODUCTION_PL;
					$this->geo_widget_api_url
					     = 'https://api-pl-points.easypack24.net/v1';
				}

				if ( self::COUNTRY_UK === $this->country ) {
					$url = self::API_URL_PRODUCTION_UK;
					$this->geo_widget_api_url
					     = 'https://api-uk-points.easypack24.net/v1';
				}
			}

			$url        = untrailingslashit( $url );
			$parsed_url = parse_url( $url );
			if ( ! isset( $parsed_url['path'] )
			     || trim( $parsed_url['path'] ) == ''
			) {
				$url .= '/v1';
			}

			return $url;
		}

		public function clear_cache() {
			$this->token   = get_option( 'easypack_token' );
			$this->api_url = $this->make_api_url();
			//
			//delete_option( 'easypack_cache_machines' );
			//delete_option( 'easypack_cache_machines_options' );
			//delete_option('easypack_cache_machines_cod_options');
			//delete_option( 'easypack_cache_machines_time' );
			//delete_option( 'easypack_cod_enabled' );
		}

		function translate_error( $error ) {
			$errors = [
				'receiver_email'     => __( 'Recipient e-mail',
					EasyPack::$text_domain ),
				'forbidden'          => __( 'forbidden' ),
				'receiver_phone'     => __( 'Recipient phone',
					EasyPack::$text_domain ),
				'address'            => __( 'Address', EasyPack::$text_domain ),
				'phone'              => __( 'Phone', EasyPack::$text_domain ),
				'email'              => __( 'Email', EasyPack::$text_domain ),
				'post_code'          => __( 'Post code',
					EasyPack::$text_domain ),
				'postal_code'        => __( 'Post code',
					EasyPack::$text_domain ),
				'default_machine_id' => __( 'Default parcel locker',
					EasyPack::$text_domain ),

				'not_an_email'             => __( 'not valid',
					EasyPack::$text_domain ),
				'invalid'                  => __( 'invalid',
					EasyPack::$text_domain ),
				'not_found'                => __( 'not found',
					EasyPack::$text_domain ),
				'invalid_format'           => __( 'invalid format',
					EasyPack::$text_domain ),
				'required, invalid_format' => __( 'required',
					EasyPack::$text_domain ),
				'too_many_characters'      => __( 'too many characters',
					EasyPack::$text_domain ),
				'Action (cancel) can not be taken on shipment with status (confirmed).'
				                           => __( 'Action (cancel) can not be taken on shipment with status (confirmed).',
					EasyPack::$text_domain ),
				'There are some validation errors. Check details object for more info.'
				                           => __( 'There are some validation errors.',
					EasyPack::$text_domain ),

				'You have not enough funds to pay for this parcel' => __( 'Can not create sticker. You have not enough funds to pay for this parcel',
					EasyPack::$text_domain ),

				'Access to this resource is forbidden'                                    => __( 'Invalid login or token',
					EasyPack::$text_domain ),
				'Sorry, access to this resource is forbidden'                             => __( 'Invalid login',
					EasyPack::$text_domain ),
				'Token is missing or invalid'                                             => __( 'Token is missing or invalid',
					EasyPack::$text_domain ),
				'Box machine name cannot be empty'                                        => __( 'Parcel Locker is empty. Please fill in this field.',
					EasyPack::$text_domain ),
				'Default parcel machine'                                                  => __( 'Default send parcel locker: ',
					EasyPack::$text_domain ),
				'The transaction can not be completed due to the balance of your account' => __( 'The transaction can not be completed due to the balance of your account',
					EasyPack::$text_domain ),
			];

			if ( isset( $errors[ $error ] ) ) {
				return $errors[ $error ];
			}

			return $error;
		}

		function get_error_array( $errors ) {
			$ret = '';
			if ( is_array( $errors ) ) {
				foreach ( $errors as $key => $value ) {
					if ( is_integer( $key ) ) {
						$ret .= $this->get_error_array( $value ) . ', ';
					} else {
						$ret .= $key . ': ' . $this->get_error_array( $value )
						        . ', ';
					}
				}
			} else {
				$ret .= $errors;
			}
			if ( $ret != '' ) {
				$ret = '<ul>' . $ret . '</ul>';
			}

			return trim( trim( $ret ), ',' );
		}

		function get_error_old( $errors ) {
			$ret = '';
			foreach ( $errors as $key => $error ) {
				$ret .= '<li>' . $this->translate_error( $key ) . ': ';
				foreach ( $error as $error_detail ) {
					if ( is_array( $error_detail ) ) {
						$ret .= $this->get_error( $error_detail );
					} else {
						$ret .= $this->translate_error( $error_detail ) . ', ';
					}
				}
				$ret = trim( trim( $ret ), ',' );
				$ret .= '</li>';
			}
			if ( $ret != '' ) {
				$ret = '<ul class="easypack_error">' . $ret . '</ul>';
			}

			return trim( trim( $ret ), ',' );
		}

		public function get_error( $errors ) {
			return $this->get_error_recursive( $errors, 10 );
		}

		/**
		 * @param      $array
		 * @param int $level
		 * @param null $key_recursive
		 *
		 * @return string
		 */
		private function get_error_recursive(
			$array,
			$level = 1,
			$key_recursive = null
		) {

			$output = '';
			if ( null !== $key_recursive
			     && ! is_numeric( $key_recursive )
			) {
				$output .= $key_recursive . ' ';
			}
			foreach ( $array as $key => $value ) {

				if ( is_array( $value ) ) {

					$output .= $this->get_error_recursive( $value, $level + 1,
						$key );
				} else {
					if ( ! is_numeric( $key ) ) {
						$value  = str_replace( '_', ' ', $value );
						$output .= $key . ': ' . $value . '<br>';
					} else {
						if ( ! is_array( $value ) ) {
							$output .= $value . '<br>';
						}
						//$output .= $key . ' ' . $key_recursive . ': ' . $value . '<br>';

					}
				}
			}

			return $output;
		}

		/**
		 * @param array $response
		 *
		 * @return bool
		 */
		private function is_binary_response( $response ) {
			if ( ! isset( $response['headers']['content-transfer-encoding'] ) ) {
				return false;
			}

			$headers = $response['headers'];
			$data    = $headers->getAll();

			return $data['content-transfer-encoding'] === 'binary';
		}

		public function post( $path, $args = [], $method = 'POST' ) {
			$url          = untrailingslashit( $this->api_url )
			                . str_replace( ' ', '%20',
					str_replace( '@', '%40', $path ) );
			$request_args = [ 'timeout' => 30, 'method' => $method ];

			$request_args['headers'] = [
				'Authorization' => 'Bearer ' . $this->token,
				'Content-Type'  => 'application/json',
			];

			$request_args['body'] = $args;
			$request_args['body'] = json_encode( $args );

			$response = wp_remote_post( $url, $request_args );

			if ( defined( 'SHIPX_API_LOG' ) ) {
				$this->addToLog(
					'POST',
					$url,
					$request_args,
					$response
				);
			}

			if ( is_wp_error( $response ) ) {
				throw new Exception( $response->get_error_message() );
			} else {

				if ( $this->is_binary_response( $response ) ) {
					return [
						'headers' => $response['headers'],

						'body' => $response['body'],
					];
				}

				$ret = json_decode( $response['body'], true );
				if ( ! is_array( $ret ) ) {
					throw new Exception( __( 'Bad API response. Check API URL',
						EasyPack::$text_domain ), 503 );
				} else {
					if ( isset( $ret['status'] ) ) {
						$errors = '';

						if ( isset( $ret['error'] )
						     && ! empty( $ret['error'] )
						) {
							if ( is_array( $ret['details'] ) ) {
								if ( count( $ret['details'] ) ) {
									$errors
										= $this->get_error( $ret['details'] );
								}
							} else {
								$errors = ': ' . $ret['details'];
							}
						} else {
							if ( isset( $ret['message'] ) ) {
								$errors
									= $this->translate_error( $ret['message'] );
							}
						}
						if ( isset( $ret['errors'] )
						     || isset( $ret['error'] )
						) {
							if ( empty( $errors ) ) {
								$errors = $ret['message'];
							}
							throw new Exception(
								str_replace( '_', ' ', $errors ),
								$ret['status'] );
						}
					}
				}

				return $ret;
			}

		}


		/**
		 * @param        $path
		 * @param array $args
		 * @param string $method
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function delete( $path, $args = [], $method = 'DELETE' ) {
			$url          = untrailingslashit( $this->api_url )
			                . str_replace( ' ', '%20',
					str_replace( '@', '%40', $path ) );
			$request_args = [ 'timeout' => 30, 'method' => $method ];

			$request_args['headers'] = [
				'Authorization' => 'Bearer ' . $this->token,
				'Content-Type'  => 'application/json',
			];

			$request_args['body'] = $args;
			$request_args['body'] = json_encode( $args );

			$response = wp_remote_post( $url, $request_args );
			if ( is_wp_error( $response ) ) {
				throw new Exception( $response->get_error_message() );
			} else {

				if ( $this->is_binary_response( $response ) ) {
					return [
						'headers' => $response['headers'],
						'body'    => $response['body'],
					];
				}

				$ret = json_decode( $response['body'], true );
				if ( ! is_array( $ret ) ) {
					throw new Exception( __( 'Bad API response. Check API URL',
						EasyPack::$text_domain ), 503 );
				} else {
					if ( isset( $ret['status'] ) ) {
						$errors = '';
						if ( isset( $ret['error'] )
						     && count( $ret['error'] )
						) {
							if ( ! empty( $ret['message'] ) ) {
								$errors
									= $this->translate_error( $ret['message'] );
								throw new Exception( $errors, $ret['status'] );
							}
							if ( is_array( $ret['details'] ) ) {
								if ( count( $ret['details'] ) ) {
									$errors
										= $this->get_error( $ret['details'] );
								}
							} else {
								$errors = ': ' . $ret['details'];
							}
						} else {
							$errors = $this->translate_error( $ret['message'] );
						}
						if ( isset( $ret['errors'] )
						     || isset( $ret['error'] )
						) {
							if ( empty( $errors ) ) {
								$errors = $ret['message'];
							}
							throw new Exception( $errors, $ret['status'] );
						}
					}
				}

				return $ret;
			}

		}


		public function put( $path, $args = [], $method = 'PUT' ) {
			return $this->post( $path, $args, 'PUT' );
		}

		public function get(
			$path,
			$args = [],
			$params = [],
			$decode_url = false
		) {
			$url = untrailingslashit( $this->api_url ) . str_replace( ' ',
					'%20',
					str_replace( '@', '%40', $path ) );

			if ( ! empty( $params ) ) {
				$newUrl = $url;
				foreach ( $params as $k => $v ) {
					$newUrl = add_query_arg( $k, $v, $newUrl );
				}

				if ( true === $decode_url ) {
					$url = preg_replace( '/\[[^\[\]]*\]/', '[]',
						urldecode( $newUrl ) );
					//$url =  urldecode( $newUrl );
				} else {
					$url = $newUrl;
				}

				//die( $url );
			}

			$request_args = [ 'timeout' => 30 ];

			$request_args['headers'] = [
				'Authorization' => 'Bearer ' . $this->token,
				'Content-Type'  => 'application/json',
			];

			$response = wp_remote_get( $url, $request_args );

			if ( defined( 'SHIPX_API_LOG' ) ) {
				$this->addToLog(
					'GET',
					$url,
					$request_args,
					$response
				);
			}

			if ( is_wp_error( $response ) ) {

				$this->authorizationError( $response->get_error_message()
				                           . ' ( Endpoint: ' . $url . ' )'
					, $response->get_error_code() );


			} else {


				if ( $this->is_binary_response( $response ) ) {
					return [
						'headers' => $response['headers'],
						'body'    => $response['body'],
					];
				}

				$ret = json_decode( $response['body'], true );
				if ( ! is_array( $ret ) ) {
					throw new Exception( __( 'Bad API response. Check API URL',
						EasyPack::$text_domain ), 503 );
				} else {
					if ( isset( $ret['status'] ) ) {
						$errors = '';
						if ( isset( $ret['error'] )
						     && ! empty( $ret['error'] )
						) {
							if ( is_array( $ret['details'] ) ) {
								if ( count( $ret['details'] ) ) {
									$errors
										= $this->get_error( $ret['details'] );
								}
							} else {
								if ( ! empty( $ret['details'] ) ) {
									$errors = ': ' . $ret['details'];
								}

								if ( ! empty( $ret['message'] ) ) {
									$errors = ': ' . $ret['message'];
								}
							}
						} else {
							if ( isset( $ret['message'] ) ) {
								$errors
									= $this->translate_error( $ret['message'] );
							}
						}
						if ( isset( $ret['errors'] )
						     || isset( $ret['error'] )
						) {
							if ( empty( $errors ) ) {
								$errors = $ret['message'];
							}
							$this->authorizationError( $errors,
								$ret['status'] );

						}
					}
				}

				return $ret;
			}

		}

		/**
		 * @param null $id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function ping( $id = null ) {
			$organizationId = ( null !== $id ) ? $id
				: get_option( 'easypack_organization_id' );
			$res            = $this->get( sprintf( '/organizations/%s',
				$organizationId ) );

			if ( ! isset( $res['error'] ) ) {
				$alerts = new Alerts();
				$alerts->add_success( 'Woocommerce Inpost: ' . __( 'New API settings connection test passed.' ) );

				update_option( 'easypack_api_login_error', '0' );
			} else {
				update_option( 'easypack_api_login_error', '1' );
			}

			return $res;
		}


		/**
		 * @param null $id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function get_organization( $id = null ) {
			$organizationId = ( null !== $id ) ? $id
				: get_option( 'easypack_organization_id' );
			$res            = $this->get( sprintf( '/organizations/%d',
				$organizationId ) );

			if ( isset( $res['error'] ) ) {
				$status = isset( $res['status'] ) ? (int) $res['status'] : 401;
				$this->authorizationError( $res['error'], $status );

				return null;
			}

			return $res;
		}

		/**
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function getServicesGlobal() {
			$res = $this->get( '/services/' );

			if ( isset( $res['error'] ) ) {
				$status = isset( $res['status'] ) ? (int) $res['status'] : 401;
				$this->authorizationError( $res['error'], $status );
			}

			return $res;
		}

		/**
		 * @param string $message
		 * @param int $status
		 *
		 * @throws Exception
		 */
		private function authorizationError( $message, $status ) {
			$errors = $this->translate_error( $message );

			$alerts = new Alerts();
			$alerts->add_error( 'Woocommerce Inpost: '
			                    . ( is_string( $errors ) ? $errors
					: serialize( $errors ) . $message . ' ( ' . $status . ' )' ) );
			//throw new Exception($errors, $status);

		}

		/**
		 * @param int $cacheMachinesTime
		 *
		 * @return bool
		 * @throws Exception
		 */
		private function isCacheOutdated( $cacheMachinesTime ) {
			$cacheTime = new \DateTime();
			$cacheTime->setTimestamp( $cacheMachinesTime );
			$now                = new \DateTime();
			$plus3days          = new \DateInterval( 'P3D' );
			$cacheTimePlus3Days = $cacheTime->add( $plus3days );

			return $cacheTimePlus3Days < $now;
		}

		/**
		 * @param bool $throw_excetion
		 *
		 * @return array|mixed|void
		 * @throws Exception
		 */
		public function machines( $throw_excetion = false ) {
			$machines = get_option( 'easypack_cache_machines' );

			if ( ! $machines
			     || true === $this->isCacheOutdated(
					(int) get_option( 'easypack_cache_machines_time' ) )
			) {
				$machines = [];
				try {
					$page     = 1;
					$response = $this->get( '/points', [],
						[ 'page' => $page, 'per_page' => 50 ] );

					$total = (int) $response['total_pages'];
					if ( ! empty( $response['items'] ) ) {
						$machines = array_merge( $machines,
							$response['items'] );


						for ( $p = $page + 1; $p ++, $p <= $total; ) {

							$response = $this->get( '/points', [],
								[ 'page' => $p, 'per_page' => 50 ] );
							if ( ! empty( $response['items'] ) ) {
								$machines = array_merge( $machines,
									$response['items'] );
							}
						}
					}

					//todo:: UK?

					$plus3days      = new \DateInterval( "P3D" );
					$nextReloadDate = new \DateTime();
					$nextReloadDate->add( $plus3days );
					update_option( 'easypack_cache_machines_time',
						$nextReloadDate->getTimestamp() );
					update_option( 'easypack_cache_machines', null );
					update_option( 'easypack_cache_machines', $machines, '' );
				} catch ( Exception $e ) {
					if ( $throw_excetion ) {
						throw $e;
					}
					if ( ! $machines ) {
						$machines = [];
					}
				}
			}

			return $machines;
		}

		/**
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function customer() {
			$organisationId = get_option( 'easypack_organization_id' );
			$response       = $this->get( sprintf( '/organizations/%d/users',
				$organisationId ) );

			/*$response
				= $this->get(sprintf('https://axa.shipx.pl.easypack24.net/v1/users/1',
				$organisationId));*/

			//$response = $this->get('/users/' . $this->login);
			//$response = $this->get('/users/');
			$customer = $response;

			return $customer;
		}


		/**
		 * @param $dispatch_point_id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 *
		 * @depecated
		 */
		public function dispatch_point( $dispatch_point_id ) {
			return $this->get( '/dispatch_points/' . $dispatch_point_id );
		}


		/**
		 * @param ShipX_Dispatch_Order_Point_Model[] $points
		 *
		 * @return array
		 */
		public function getDispatchPointsOptions( $points ) {
			if ( empty( $points ) ) {
				return [];
			}

			$return = [];

			foreach ( $points as $point ) {
				$return[ $point->getId() ] = $point->getName();
			}

			return $return;
		}

		/**
		 * @param $args
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function dispatch_order( $args ) {
			$organisationId = get_option( 'easypack_organization_id' );
			$response
			                = $this->post( sprintf( '/organizations/%d/dispatch_orders',
				$organisationId ), $args );

			return $response;
		}

		/**
		 * @param $args
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function cancel_dispatch_order( $order_id ) {
			$organisationId = get_option( 'easypack_organization_id' );
			$response
			                = $this->delete( sprintf( '/organizations/%d/dispatch_orders',
				$organisationId ),
				$order_id );

			return $response;
		}

		public function customer_parcel_create( $args ) {
			$organizationId = get_option( 'easypack_organization_id' );
			$response
			                = $this->post( sprintf( '/organizations/%d/shipments',
				$organizationId ), $args );


			return $response;
		}

		/**
		 * @param $id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		/*public function customer_parcel_get_by_id($id)
		{
			$organizationId = get_option('easypack_organization_id');
			$response = $this->get(sprintf('/organizations/%d/shipments',
				$organizationId),
				[], ['id' => $id]);

			return $response;
		}*/

		public function customer_parcel_get_by_id( $id ) {

			$response = $this->get( sprintf( '/shipments/%d',
				$id ) );

			return $response;
		}

		/**
		 * @return string
		 * @throws Exception
		 */
		public function get_statuses() {
			$response = $this->get( '/statuses' );

			return $response;
		}

		/**
		 * @param $parcel_id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function customer_parcel_cancel( $parcel_id ) {
			$response = $this->delete( '/shipments/' . $parcel_id );

			return $response;
		}

		public function customer_parcel_pay( $parcel_id ) {
			$response = $this->post( '/parcels/' . $parcel_id . '/pay', $args );

			return $response;
		}


		public function customer_parcel_sticker( $parcel_id ) {
			return $this->get( '/shipments/' . $parcel_id . '/label',
				[ 'format' => 'Pdf' ] );
		}


		/**
		 * @param $shipment_ids
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function customer_shipments_labels( $shipment_ids ) {
			$organizationId = get_option( 'easypack_organization_id' );
			$labelFormat    = get_option( 'easypack_label_format' );

			$args = [
				'format'       => 'pdf',
				'shipment_ids' => $shipment_ids,
				'type'         => $labelFormat === 'A4' ? 'normal' : 'A6',
			];

			return $this->get( sprintf( '/organizations/%d/shipments/labels',
				$organizationId ), [], $args, true );
		}

		/**
		 * @param $shipment_ids
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function customer_shipments_return_labels( $shipment_ids ) {
			$organizationId = get_option( 'easypack_organization_id' );
			$args           = [
				'format'       => 'pdf',
				'shipment_ids' => $shipment_ids,
			];

			return $this->get( sprintf( '/organizations/%d/shipments/return_labels',
					$organizationId )
				, []
				, $args
				, true
			);
		}

		/**
		 * @param $dispatch_order_id
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function dispatch_order_pdf( $dispatch_order_id ) {
			$organizationId = get_option( 'easypack_organization_id' );
			$args           = [
				'format' => 'Pdf',
			];

			return $this->get( sprintf( '/organizations/%d/dispatch_orders/%d/printout',
					$organizationId, $dispatch_order_id )
				, null
				, $args
				, true
			);
		}

		public function customer_parcel( $parcel_id ) {
			$response = $this->get( '/parcels/' . $parcel_id );
			$parcel   = $response;

			return $parcel;
		}

		public function dispatch_points_dispatch_orders(
			$dispatch_point,
			$args
		) {
			$response = $this->put( '/dispatch_point/' . $dispatch_point
			                        . '/dispatch_orders' );

			return $response;
		}

		/**
		 * @return mixed
		 * @deprecated
		 */
		public function api_country() {
			return $this->getCountry();
		}

		public function validate_phone( $phone ) {

			if ( $this->getCountry() == EasyPack_API::COUNTRY_UK ) {
				if ( preg_match( "/\A\d{10}\z/", $phone ) ) {
					return true;
				} else {
					return __( 'Invalid phone number. Valid phone number must contains 10 digits.',
						EasyPack::$text_domain );
				}
			}
			if ( $this->getCountry() == EasyPack_API::COUNTRY_PL ) {
				if ( preg_match( "/\A[1-9]\d{8}\z/", $phone ) ) {
					return true;
				} else {
					return __( 'Invalid phone number. Valid phone number must contains 9 digits and must not begins with 0.',
						EasyPack::$text_domain );
				}
			}

			return __( 'Invalid phone number.', EasyPack::$text_domain );

		}

		/**
		 * @param $shipments
		 *
		 * @return array|mixed|object
		 * @throws Exception
		 */
		public function calculate_shipments( $shipments ) {
			$organizationId = get_option( 'easypack_organization_id' );
			$response
			                = $this->post( sprintf( '/organizations/%d/shipments/calculate'
				, $organizationId
			),
				$shipments
			);

			return $response;
		}

		/**
		 * @return mixed
		 */
		public function getEnvironment() {
			return $this->environment;
		}

		/**
		 * @return mixed
		 */
		public function getCountry() {
			return $this->country;
		}

		/**
		 * @return mixed
		 */
		public function getApiUrl() {
			return $this->api_url;
		}

		/**
		 * @return mixed
		 */
		public function getGeoWidgetApiUrl() {
			return $this->geo_widget_api_url;
		}

		/**
		 * @return string
		 */
		public function getGeoWidgetApiCssUrl() {
			return $this->geo_widget_api_css_url;
		}

		/**
		 * @param string $method
		 * @param string $url
		 * @param array $request
		 * @param array $response
		 */
		public function addToLog( $method, $url, $request, $response ) {
			$file = WOO_INPOST_PLUGIN_DIR
			        . DIRECTORY_SEPARATOR
			        . 'log-inpost.txt';

			$line
				= sprintf( "******************\n\n%s\n%s\nURL:%s\nREQUEST:\n%s\nRESPONSE:\n%s\n",
				$method,
				date( "Y-m-d H:i:s", time() ),
				$url,
				preg_replace( '/[\x00-\x1F\x7F]/u', '', serialize( $request ) ),
				//remove non printable characters
				preg_replace( '/[\x00-\x1F\x7F]/u', '', serialize( $response ) )
			);

			file_put_contents( $file, $line, FILE_APPEND );
		}


		/**
		 * @return string
		 */
		private function getWpInstallation() {
			$full_path = getcwd();
			$ar        = explode( "wp-", $full_path );

			return $ar[0];
		}

		/**
		 * @param $message
		 */
		public function printApiError( $message, $description = null ) {
			printf( '<div class="error"><p>%s<b>%s</b><br/></p></div>'
				,
				null === $description
					? __( 'InPost API configuration error:  ',
					EasyPack::$text_domain )
					: $description
				, $message
			);
		}
	}


endif;

